#!/usr/bin/perl
#
# SILABAS.PL Ultima atualizacao : 05/06/2001
# ltima alteracao 5/8/2003
# Criado por Miguel Galves (miguel.galves@ic.unicamp.br)
# Programa que etiqueta palavras segundo 
# numero de silabas e posicao do acento 
#

# ##########################################################################

sub print_stat {

	my ($len,$type) = @_;
	print STAT "$len $type\n";

}

# ########################################################################

# Sub rotina que verifica a presenca de enclise na palavra
# retornando um valor inteiro que correspondera a quanto
# se devera acrescentar para se saber a posicao correta do acento
sub verify_enc {

    my ($token) = @_;
    pos $token = 0;
    if ($token =~m/(\w)+-(me|te|se|lhe|o|a|nos|vos|lho|lha|mo|ma|to|ta)s?$/i) 
	{
		return 1;
	}
	if ($token =~ m/(\w)+-(no-lo|no-la|vo-lo|vo-la|se-lhe)s?$/i)
	{
		return 2;
	}
	return 0;

} # sub &verify_enc


# ##########################################################################

# sub rotina que aplica regras de acentuacao para
# palavras que nao possuem acento grafico
sub apply_rules {

    my ($token) = @_;
	my $enc = &verify_enc($token);
	$sub_token = $token;
	# Verifica se existe enclise na palavra analisada
	# caso exista, separa a substring anterior aa enclise para analise
	if ($enc>0)
	{
		$sub_token = substr($token,0,index($token,"-"));
	}
	# Aplica as regras para palavras nao acentuadas
    if ($sub_token =~ m/(a|e|o|as|os|es|em|ens|am)$/i)
	{
		if ($enc == 0) {return "PAR";}
		else {return "PRO";}
    }
    else 
	{
		if ($enc == 0) {return "OXI";}
		if ($enc == 1) {return "PAR";}
		else {return "PRO";}
    }
} # sub &apply_rules


# ########################################################################

# sub rotina que aplica regras de acentuacao 
# para palavras que possuem acento grafico.
# Para isso, extrai uma substring do token inicial que inicia
# na primeira letra apos o acento ou ditongo tonico (se existir)
# contando o numero de silabas da substring.
sub find_accent {

    my $sub_sil = 0;
    my ($token, $num_sil, $pos_acent, $token_acent) = @_;
    my $sub_token = "";
	
	# Extrai a substring iniciada apos o acento grafico.
	# Verifica se  a letra que segue a vogal acentuada nao e i ou u 
	# ou se a vogal acentuada nao e til
    if (($token !~ m/$token_acent([iu])/ig) && ($token_acent ne "" && 
$token_acent ne "")) 
	{
		$sub_token = substr ($token, $pos_acent+1, 
                length($token)-($pos_acent+1));
    }
    else 
	{  
        $sub_token = substr ($token, $pos_acent+2, 
        length($token)-($pos_acent+2));
    }
	# Conta o numero de vogais, e elimina os ditongos
    while ($sub_token =~ m/[aeiouy]/gi) 
	{
		++ $sub_sil;
    }
    if ($sub_token =~ m/([aeiouy])([aeiouy])/gi) 
	{
		-- $sub_sil;
    }
	# Adiciona o numero de silabas da enclise
	$sub_sil = $sub_sil+$enc;
    if ($sub_sil == 0) { print RULES "Oxitona acentuada \n"; return "OXI";}
    if ($sub_sil == 1) { print RULES "Paroxitona acentuada \n"; return "PAR";}
    if ($sub_sil >= 2) { print RULES "Proparoxitona acentuada \n"; 
        return "PRO";}
    
} # sub &find_accent


# ########################################################################


# sub rotina que faz o trabalho mais importante
# do script : determina o numero de silabas.
sub parse_token {
    
    my $num_sil = 0;
    my $pos_acent = -1;
    my $acento = 0;
    my ($token) = @_ ;
    my $tag = "";
    my $sub_tag = "";
    my $token_acent;
    
    # procura as vogais  
    while ($token =~ m/[aeiouy]/gi) 
    {
	++ $num_sil;
    }
    
    # verifica a existencia de uma vogal acentuada
    # guardando sua posicao caso exista e verificando casos
    # onde e necessario reduzir o numero de silabas.
    foreach $acent (@acents)
    {
	if (($pos_acent = index($token, $acent))>-1)
	{
	    $token_acent = $acent;
	    $acento = 1;
	    $num_sil++;
	    # Verifica se a vogal acentuada e um til, e se esta e precedida
	    # por o ou e, reduzindo o numero de vogais caso verdade
	    if ($acent == '' || $acent == '')
	    {
		if ($token =~ m/$acent[oe]/ig)  
		{
		    -- $num_sil;
		}		
	    } 
	    # verifica se a vogal acentuada nao 
	    # pertence a um ditongo aberto tonico
	    else
	    {
		if ($token =~ m/$acent([iu])/ig)  
		{
		    -- $num_sil; 
		}
	    }
	    last;
	}
    }
    
    # ####################################################################
    
    pos $token = 0; # Zera o ponteiro de posicao de $token
    
    # Bloco que analisa grupos de duas vogais
    # seguidas identificando como ditongos ou hiatos
    while ($token =~ m/(\w?(([aeiouy])([aeiouy])))/gi) 
    {
	if ($1 eq "que" || $1 eq "qua" || $1 eq "qui" || $1 eq "quo"  || 
	    $1 eq "gue" || $1 eq "gua" || $1 eq "gui" || $1 eq "guo")
	{
	    -- $num_sil;
	}
	else
	{
	    if (($acento==1 && index($token,$1, $pos_acent)>-1) ||
		(($2 ne "ia" && $2 ne "uo" && $2 ne "ua" && $2 ne "ae" 
                  && $2 ne "oa" && $2 ne "ea" 
		  && $2 ne "ie" && $2 ne "io" && $2 ne "oe" && $2 ne "ia" 
                  && $2 ne "eo" && $3 ne $4) 
		  && ($acento==0||($acento==1 
                  && index($token,$2)<$pos_acent))))
	    {
		-- $num_sil;
	    }
	}	
    }
    # Verifica a existencia de hiatos seguidos de l,m,n,r z que nao 
    #  iniciam vogal
    if (($acento == 0) && ($token =~ m/[^qg][aeiou](i|u)(l|m|n|r|z)
       ([^aeiouh]|$)/i)) 
    {
	++ $num_sil;
    }
    # Verifica a existencia de hiatos seguidos de nh
    if (($acento == 0) && ($token =~ m/([aeiou])(i|u)(nh)/i)) 
    {
	++ $num_sil;
    }
	
    # ####################################################################
	
    # Bloco que cria a etiqueta final...
	
    # Tratamento de erro :
    # caso o numero de silabas ao final da analise seja
    # inferior a 1, indicar o erro e terminar analise.
	if ($num_sil<1)
	{
	    return $token;
	}
	# Criacao da etiqueta primaria paa palavras com mais de uma silaba
	if ($num_sil == 2) {$tag = "DIS";}
	if ($num_sil == 3) {$tag = "TRI";}
	if ($num_sil >= 4) {$tag = "POL$num_sil";}
	# Criacao da etiqueta secundaria para palavras de mais de uma silaba.
	# Verifica se a palava possue ou nao acento e chama a rotina 
        # apropriada
	if ($num_sil >1) 
	{
	    if ($acento == 0) 
	    {
	    	$sub_tag = &apply_rules($token);
	    }
	    else 
	    {
	    	$sub_tag = &find_accent($token, $num_sil, $pos_acent, 
                $token_acent);
	    }
	    $tag = "$tag-$sub_tag";
	    #&print_stat($num_sil,$sub_tag);
	    
    if (($num_sil == 2) && ($sub_tag eq "PAR") && ($pal_fon == 0))
	       { print BIN ("#1.0");print BIN ("#\n");
	         $pal_fon =1;}
    else {if (($num_sil == 2) && ($sub_tag eq "PAR") && ($pal_fon == 1))
	     {print BIN ("#3.0");print BIN ("#\n");}}

    if (($num_sil == 2) && ($sub_tag eq "OXI") && ($pal_fon == 0))
	          { print BIN ("#0.1");print BIN ("#\n"); 
		    $pal_fon=1;}
    else{ if (($num_sil == 2) && ($sub_tag eq "OXI") && ($pal_fon == 1))
                  { print BIN ("#2.1");print BIN ("#\n");}}

   if (($num_sil == 3)&&($sub_tag eq "PRO") && ($pal_fon==0))
	               { print BIN ("#1.0.0");print BIN ("#\n");
			 $pal_fon=1;}
   else{ if (($num_sil == 3)&&($sub_tag eq "PRO") && ($pal_fon==1))
                       { print BIN ("#3.0.0");print BIN ("#\n");}}

   if (($num_sil == 3) && ($sub_tag eq "PAR") && ($pal_fon==0))
	    	{ print BIN ("#0.1.0");print BIN ("#\n");
		  $pal_fon=1;}
   else{ if (($num_sil == 3) && ($sub_tag eq "PAR") && ($pal_fon==1))
                { print BIN ("#2.1.0");print BIN ("#\n");}}

   if (($num_sil == 3) && ($sub_tag eq "OXI")&& ($pal_fon == 0))
	        { print BIN ("#0.0.1");print BIN ("#\n");
		  $pal_fon=1;}
   else{ if (($num_sil == 3) && ($sub_tag eq "OXI")&& ($pal_fon == 1))
                { print BIN ("#2.0.1");print BIN ("#\n");}}
	   	
   if (($num_sil == 4) && ($sub_tag eq "PAR") && ($pal_fon == 0))
	    	{ print BIN ("#0.0.1.0");print BIN ("#\n");
		  $pal_fon=1;}
   else{ if (($num_sil == 4) && ($sub_tag eq "PAR") && ($pal_fon == 1))
                { print BIN ("#2.0.1.0");print BIN ("#\n");}}

   if (($num_sil == 4) && ($sub_tag eq "PRO") && ($pal_fon == 0))
	    	{ print BIN ("#0.1.0.0");print BIN ("#\n");
		  $pal_fon=1;}
   else{ if (($num_sil == 4) && ($sub_tag eq "PRO") && ($pal_fon == 1))
                { print BIN ("#2.1.0.0");print BIN ("#\n");}}

   if (($num_sil == 4) && ($sub_tag eq "OXI") && ($pal_fon==0))
	    { print BIN ("#0.0.0.1");print BIN ("#\n");
	      $pal_fon=1;}
   else{ if (($num_sil == 4) && ($sub_tag eq "OXI") && ($pal_fon==1))
            { print BIN ("#2.0.0.1");print BIN ("#\n");}}

   if (($num_sil == 5) && ($sub_tag eq "PRO")&& ($pal_fon==0))
	    { print BIN ("#0.0.1.0.0");print BIN ("#\n");
	      $pal_fon=1;}
   else { if (($num_sil == 5) && ($sub_tag eq "PRO")&& ($pal_fon==1))
            { print BIN ("#2.0.1.0.0");print BIN ("#\n");}}

   if (($num_sil == 5) && ($sub_tag eq "PAR") &&($pal_fon == 0))
	    { print BIN ("#0.0.0.1.0");print BIN ("#\n");
	      $pal_fon=1;}
   else{ if (($num_sil == 5) && ($sub_tag eq "PAR") &&($pal_fon == 1))
            { print BIN ("#2.0.0.1.0");print BIN ("#\n");}}

   if (($num_sil == 5) && ($sub_tag eq "OXI")&& ($pal_fon==0))
	    { print BIN ("#0.0.0.0.1");print BIN ("#\n");
	      $pal_fon=1;}
   else { if (($num_sil == 5) && ($sub_tag eq "OXI")&& ($pal_fon==1))
            { print BIN ("#2.0.0.0.1");print BIN ("#\n");}}

if (($num_sil == 6) && ($sub_tag eq "PRO")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.1.0.0");print BIN ("#\n"); 
              $pal_fon=1;} 
   else { if (($num_sil == 6) && ($sub_tag eq "PRO")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.1.0.0");print BIN ("#\n");}} 

   if (($num_sil == 6) && ($sub_tag eq "PAR") &&($pal_fon == 0)) 
            { print BIN ("#0.0.0.0.1.0"); print BIN ("#\n");
              $pal_fon=1;} 
   else{ if (($num_sil == 6) && ($sub_tag eq "PAR") &&($pal_fon == 1)) 
            { print BIN ("#2.0.0.0.1.0");print BIN ("#\n");}} 

   if (($num_sil == 6) && ($sub_tag eq "OXI")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.0.0.1"); print BIN ("#\n");
              $pal_fon=1;} 
   else { if (($num_sil == 6) && ($sub_tag eq "OXI")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.0.0.1");print BIN ("#\n");}} 

   if (($num_sil == 7) && ($sub_tag eq "PRO")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.0.1.0.0"); print BIN ("#\n");
              $pal_fon=1;} 
   else { if (($num_sil == 7) && ($sub_tag eq "PRO")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.0.1.0.0");print BIN ("#\n");}} 

   if (($num_sil == 7) && ($sub_tag eq "PAR") &&($pal_fon == 0)) 
            { print BIN ("#0.0.0.0.0.1.0"); print BIN ("#\n");
              $pal_fon=1;} 
   else{ if (($num_sil == 7) && ($sub_tag eq "PAR") &&($pal_fon == 1)) 
            { print BIN ("#2.0.0.0.0.1.0");print BIN ("#\n");}} 

   if (($num_sil == 7) && ($sub_tag eq "OXI")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.0.0.0.1"); print BIN ("#\n");
              $pal_fon=1;} 
   else { if (($num_sil == 7) && ($sub_tag eq "OXI")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.0.0.0.1");print BIN ("#\n");}} 

   if (($num_sil == 8) && ($sub_tag eq "PRO")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.0.0.1.0.0"); print BIN ("#\n");
              $pal_fon=1;} 
   else { if (($num_sil == 8) && ($sub_tag eq "PRO")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.0.0.1.0.0");print BIN ("#\n");}} 

   if (($num_sil == 8) && ($sub_tag eq "PAR") &&($pal_fon == 0)) 
            { print BIN ("#0.0.0.0.0.0.1.0"); print BIN ("#\n");
              $pal_fon=1;} 
   else{ if (($num_sil == 8) && ($sub_tag eq "PAR") &&($pal_fon == 1)) 
            { print BIN ("#2.0.0.0.0.0.1.0 ");print BIN ("#\n");}} 

   if (($num_sil == 8) && ($sub_tag eq "OXI")&& ($pal_fon==0)) 
            { print BIN ("#0.0.0.0.0.0.0.1"); print BIN ("#\n");
              $pal_fon=1;} 
   else { if (($num_sil == 8) && ($sub_tag eq "OXI")&& ($pal_fon==1)) 
            { print BIN ("#2.0.0.0.0.0.0.1");print BIN ("#\n");}} 

         
} 
	# Caso especial : monossilabos.
	else
   	{  
	    # Bloco executado caso o monossilabo nao possua acento grafico
	    if ($acento == 0)
	    {
		if ($token =~ m/^(mo|to|me|te|se|na|nas|no|lhe|o|a|nos
				  |vos|lho|lha|um|uns|sem|com|me|mas|ou|
				  |por|de||ao|do|da|que|se|e|em|ens|per
				  |no-lo|no-la|vo-lo|vo-la)s?$/i) 
		{		 
		    $tag = "MA";
	            if ($pal_fon==1)
	             {print BIN ("#2");print BIN ("#\n");
	              $pal_fon=0;}
	            else{ { print BIN ("#0");print BIN ("#\n");}}

		}
		else 
		{
		    $tag = "MT";
		    if ($pal_fon==1)
                    {print BIN ("#3");print BIN ("#\n");}
	            else{ { print BIN ("#1");print BIN ("#\n");
			    $pal_fon = 1;}}
		}
		# Caso ao monissilabo possua acento grafico
	    }
	    else {$tag = "MT";
                  if ($pal_fon==1)
                    {print BIN ("#3");print BIN ("#\n");}
                  else{ { print BIN ("#1");print BIN ("#\n");
                            $pal_fon = 1;}}

                  }
	    #&print_stat(1,$tag);

  	}
	return "/$tag";
    } # sub &parse_token
    
    
# ######################################################################
# Bloco principal do programa

# abre os arquivos de dados
print "Silabas 1.5 - Por Miguel Galves.\n";
print "(miguel.galves\@ic.unicamp.br)\n";
$fin = @ARGV[0];

if (rindex($fin, '.')!= -1)
{
	$sub_fin = substr ($fin, 0, rindex($fin, '.'));
}
else {$sub_fin = $fin;}

# $fout = "saida.slb";
# $fstat = "saida.stat"; 
$fbin = "saida.bin";

if (!open(IN, "entrada.txt"))
{
	print "Arquivo $fin nao existente\n";
	exit;
}
print "Arquivo de entrada : $fin \n";
# open (OUT, ">$fout");
# open (STAT, ">$fstat");
open(BIN, ">$fbin");

# inicializa variaveis globais
@acents = split (//, "");
# le todas as linhas do arquivo de entrada
# armazenando as em um vetor
# @lines = <IN>;
# separa cada linha em tokens delimitados por
#  .,;()-?!<>
print "Iniciando analise do arquivo $fin...";
$contador = 0;
$pal_fon = 1;
# foreach $line (@lines)
while ($line = <IN>)
{
    $line =~ tr/<.+>?.+<?>//;
    @tokens = split (/([\s\.\?!,;:()])/, $line); 
    # quebra uma linha em varios tokens
    foreach $token (@tokens)
    {   
	if ($token =~ m/[^\s\.\!\?(),;:]/i)
	{
	    $contador++;
	    $new_token = lc $token;
	   # print BIN ($token);
	    $ext = &parse_token ($new_token); # Analise do token
	    
	    $token= $token.$ext;
	}
	else 
	{			
	    if ($token eq "." || $token eq "!" || $token eq "?")
	      {
                print BIN ("4\n" );
#o seguinte contador informa o numero acumulado de palavras registradas ate o 4
		#print BIN ("$contador\n");
               # &print_stat(0,$token);
	    }
	}
    }
    # Imprime o resultado da analise no arquivo de saida
    #print OUT join("", @tokens);
}
#$random=int(rand(10));
#print BIN ("$random");

close (IN);
close (OUT);
# close (STAT);

print "Analise terminada. $contador palavras analisadas.\n";

